@@ -1239,6 +1239,9 @@ hildon_desktop_home_item_propagate_button (HildonDesktopHomeItem *item,
   GdkEventButton       *event;
   GdkWindow            *child;
 
+  if (!priv->last_click_event)
+    return;
+
   child = hildon_desktop_home_item_get_window_at (item,
                                                   priv->last_click_event->x,
                                                   priv->last_click_event->y,
@@ -1275,7 +1278,6 @@ hildon_desktop_home_item_propagate_button (HildonDesktopHomeItem *item,
     gdk_event_put ((GdkEvent *)crossing_event);
 
     ((GdkEventAny *)event)->window = g_object_ref (child);
-    ((GdkEventAny *)event)->type = GDK_BUTTON_PRESS;
     gdk_event_put ((GdkEvent *)event);
   }
 
@@ -1325,13 +1327,48 @@ static gboolean
 hildon_desktop_home_item_button_press_event (GtkWidget *w,
                                              GdkEventButton   *event)
 {
+  HildonDesktopHomeItem          *item;
   HildonDesktopHomeItemPriv      *priv;
 
   priv = HILDON_DESKTOP_HOME_ITEM_GET_PRIVATE (HILDON_DESKTOP_HOME_ITEM (w));
+  item = HILDON_DESKTOP_HOME_ITEM (w);
 
+  /* ignore events which do not occur on our event windows (typically
+   * events we propagated to the bin */
   if (event->window != priv->event_window &&
-      event->window != priv->resize_handle_window)
+      event->window != priv->resize_handle_window &&
+      event->window != priv->close_button_window)
+  {
     return FALSE;
+  }
+
+  if (event->type != GDK_BUTTON_PRESS)
+  {
+    /* In case of double click, we can propagate directly */
+
+    if (priv->click_timeout)
+    {
+      g_source_remove (priv->click_timeout);
+      priv->click_timeout = 0;
+
+      /* First propagate the pending button press */
+      hildon_desktop_home_item_propagate_button (item,
+                                                 TRUE,
+                                                 FALSE);
+    }
+
+    gdk_event_free ((GdkEvent *)priv->last_click_event);
+
+    /* ... then propagate the double-click */
+    priv->last_click_event =
+        (GdkEventButton *)gdk_event_copy ((GdkEvent *)event);
+    hildon_desktop_home_item_propagate_button (item,
+                                               TRUE,
+                                               FALSE);
+
+    /* The button release will be propagated in the button release handler */
+    return TRUE;
+  }
 
   /* Check if we clicked the close button */
   if (event->window == priv->close_button_window)
@@ -1349,7 +1386,7 @@ hildon_desktop_home_item_button_press_event (GtkWidget *w,
 
       if (event->window == priv->resize_handle_window)
         {
-          hildon_desktop_home_item_set_state (HILDON_DESKTOP_HOME_ITEM (w),
+          hildon_desktop_home_item_set_state (item,
                                               HILDON_DESKTOP_HOME_ITEM_STATE_RESIZING,
                                               event);
         }
@@ -1386,7 +1423,9 @@ hildon_desktop_home_item_button_release_event (GtkWidget       *widget,
 
   if (event->window != priv->event_window &&
       event->window != priv->resize_handle_window)
+  {
     return FALSE;
+  }
 
   if (event->window == priv->event_window)
   {
@@ -1401,6 +1440,12 @@ hildon_desktop_home_item_button_release_event (GtkWidget       *widget,
     }
     else if (!priv->moving_threshold)
     {
+      if (priv->last_click_event)
+        gdk_event_free ((GdkEvent *)priv->last_click_event);
+
+      priv->last_click_event =
+          (GdkEventButton *)gdk_event_copy ((GdkEvent *)event);
+
       hildon_desktop_home_item_propagate_button (item,
                                                  FALSE,
                                                  TRUE);
