@@ -1063,14 +1063,14 @@ hildon_desktop_x_event_filter (GdkXEvent *xevent,
     HILDON_DESKTOP_PANEL_EXPANDABLE (_panel);
 
   XEvent *e = (XEvent*)xevent;
-
+  
   if (e->type == ClientMessage)
   {
     if (e->xclient.message_type == panel->priv->tray_opcode &&
         e->xclient.data.l[1] == SYSTEM_TRAY_REQUEST_DOCK)
     {
       gchar *id = g_strdup_printf ("XEmbed:%d",(GdkNativeWindow)e->xclient.data.l[2]);
-	    
+      
       if (g_hash_table_lookup (panel->priv->items, id) != NULL)
       {
 	g_free (id);
@@ -1078,7 +1078,7 @@ hildon_desktop_x_event_filter (GdkXEvent *xevent,
       }
 
       g_free (id);
-
+	
       if (panel->priv->n_systray_applets <= (panel->priv->items_p_row - 1))
         hildon_desktop_panel_embed_applet (panel,(Window)e->xclient.data.l[2]);
       
@@ -1086,7 +1086,7 @@ hildon_desktop_x_event_filter (GdkXEvent *xevent,
     }
   }
 
-  return TRUE;
+  return GDK_FILTER_CONTINUE;
 }
 
 static gboolean 
@@ -1147,7 +1147,8 @@ hildon_desktop_panel_embed_applet (HildonDesktopPanelExpandable *panel, Window w
   hildon_desktop_item_socket_add_id (HILDON_DESKTOP_ITEM_SOCKET (sb_socket), 
 		  		     wid);
 
-
+  panel->priv->n_systray_applets++;
+  
   g_free (id);
   g_free (name);
 }
