@@ -25,6 +25,7 @@
 
 #include "hildon-desktop-panel.h"
 #include "hildon-desktop-panel-item.h"
+#include "hildon-desktop-container.h"
 #include "statusbar-item.h"
 
 #include <gtk/gtkhbox.h>
@@ -425,8 +426,11 @@ hildon_desktop_panel_refresh_items_status (HildonDesktopPanel *panel)
   GList *children = NULL, *l;
 
   g_return_if_fail (panel && HILDON_DESKTOP_IS_PANEL (panel));
- 
-  children = gtk_container_get_children (GTK_CONTAINER (panel));
+
+  if (HILDON_DESKTOP_IS_CONTAINER (panel))
+    children = hildon_desktop_container_get_children (HILDON_DESKTOP_CONTAINER (panel));
+  else	    
+    children = gtk_container_get_children (GTK_CONTAINER (panel));
 
   for (l = children; l != NULL; l = g_list_next (l))
   {	  
