@@ -56,7 +56,8 @@ enum
 
 static gint signals[N_SIGNALS];
 
-#define MENU_SCROLL_TIMEOUT 40
+#define MENU_SCROLL_TIMEOUT          50
+#define MENU_SCROLL_INITIAL_TIMEOUT  200
 
 struct _HildonDesktopPopupMenuPrivate
 {
@@ -109,7 +110,6 @@ static gboolean hildon_desktop_popup_menu_press_event (GtkWidget *widget, GdkEve
 static gboolean hildon_desktop_popup_menu_release_event (GtkWidget *widget, GdkEventButton *event);
 static gboolean hildon_desktop_popup_menu_key_press_event (GtkWidget *widget, GdkEventKey *event);
 static void hildon_desktop_popup_menu_select_next_prev_item (HildonDesktopPopupMenu *menu, gboolean next);
-static void hildon_desktop_popup_menu_scroll_cb (GtkWidget *widget, HildonDesktopPopupMenu *menu);
 static void hildon_desktop_popup_menu_scroll_start (GtkWidget *widget, HildonDesktopPopupMenu *menu);
 static void hildon_desktop_popup_menu_scroll_stop (GtkWidget *widget, HildonDesktopPopupMenu *menu);
 static void hildon_desktop_popup_menu_adjustment_changed (GtkAdjustment *adj, HildonDesktopPopupMenu *menu);
@@ -191,7 +191,7 @@ hildon_desktop_popup_menu_class_init (HildonDesktopPopupMenuClass *menu_class)
                                            "resize-parent",
                                            "Whether resize or not parent window of menu",
                                             TRUE,
-                                            G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                            G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
   g_object_class_install_property (object_class,
                                    PROP_POPUP_PARENT,
@@ -248,6 +248,19 @@ hildon_desktop_popup_menu_paint_pane (GtkWidget      *widget,
   return TRUE;
 }
 
+static gboolean
+hildon_desktop_popup_menu_item_is_selectable (GtkWidget *menu_item)
+{
+  if ((!GTK_BIN (menu_item)->child &&
+      GTK_IS_MENU_ITEM (menu_item)) ||
+      GTK_IS_SEPARATOR_MENU_ITEM (menu_item) ||
+      !GTK_WIDGET_IS_SENSITIVE (menu_item) ||
+      !GTK_WIDGET_VISIBLE (menu_item))
+    return FALSE;
+
+  return TRUE;
+}
+	
 static GObject *
 hildon_desktop_popup_menu_constructor (GType gtype,
                                        guint n_params,
@@ -306,11 +319,6 @@ hildon_desktop_popup_menu_constructor (GType gtype,
 		"can-focus", FALSE,
 		NULL);
   
-  g_signal_connect (menu->priv->scroll_down,
-		    "clicked",
-		    G_CALLBACK (hildon_desktop_popup_menu_scroll_cb),
-		    (gpointer)menu);
-
   g_signal_connect (menu->priv->scroll_down,
 		    "pressed",
 		    G_CALLBACK (hildon_desktop_popup_menu_scroll_start),
@@ -321,11 +329,6 @@ hildon_desktop_popup_menu_constructor (GType gtype,
 		    G_CALLBACK (hildon_desktop_popup_menu_scroll_stop),
 		    (gpointer)menu);
 
-  g_signal_connect (menu->priv->scroll_up,
-                    "clicked",
-                    G_CALLBACK (hildon_desktop_popup_menu_scroll_cb),
-                    (gpointer)menu);		    
-
   g_signal_connect (menu->priv->scroll_up,
 		    "pressed",
 		    G_CALLBACK (hildon_desktop_popup_menu_scroll_start),
@@ -551,7 +554,8 @@ hildon_desktop_popup_menu_motion_notify (GtkWidget      *widget,
 
     if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
     {
-      if (menu->priv->selected_item != GTK_MENU_ITEM (l->data))
+      if (hildon_desktop_popup_menu_item_is_selectable (GTK_WIDGET (l->data)) &&
+          menu->priv->selected_item != GTK_MENU_ITEM (l->data))
       {
         gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
         gtk_item_select (GTK_ITEM (l->data));
@@ -588,6 +592,16 @@ hildon_desktop_popup_menu_adjustment_changed (GtkAdjustment *adj,
   hildon_desktop_popup_menu_update_scroll_buttons (menu);
 }
 
+static void
+hildon_desktop_popup_menu_remove_scroll_timeout (HildonDesktopPopupMenu *menu)
+{
+  if (menu->priv->scroll_id > 0)
+  {
+    g_source_remove (menu->priv->scroll_id);
+    menu->priv->scroll_id = 0;
+  }
+}
+	
 static gboolean 
 hildon_desktop_popup_menu_scroll_step (gpointer data)
 {
@@ -619,6 +633,20 @@ hildon_desktop_popup_menu_scroll_step (gpointer data)
   return TRUE;
 }
 
+static gboolean
+hildon_desktop_popup_menu_scroll_initial_step (gpointer data)
+{
+  HildonDesktopPopupMenu *menu = HILDON_DESKTOP_POPUP_MENU (data);
+
+  hildon_desktop_popup_menu_remove_scroll_timeout (menu);
+  
+  menu->priv->scroll_id = g_timeout_add (MENU_SCROLL_TIMEOUT, 
+					 hildon_desktop_popup_menu_scroll_step, 
+					 menu);
+
+  return FALSE;
+}
+
 static void 
 hildon_desktop_popup_menu_scroll_start (GtkWidget *widget, HildonDesktopPopupMenu *menu)
 {
@@ -627,26 +655,17 @@ hildon_desktop_popup_menu_scroll_start (GtkWidget *widget, HildonDesktopPopupMen
   else
     menu->priv->scroll_dir = 1;
 
-  menu->priv->scroll_id = 
-    g_timeout_add (MENU_SCROLL_TIMEOUT, hildon_desktop_popup_menu_scroll_step, menu);
+  hildon_desktop_popup_menu_scroll_step (menu);
+
+  menu->priv->scroll_id = g_timeout_add (MENU_SCROLL_INITIAL_TIMEOUT, 
+					 hildon_desktop_popup_menu_scroll_initial_step, 
+					 menu);
 }
 
 static void 
 hildon_desktop_popup_menu_scroll_stop (GtkWidget *widget, HildonDesktopPopupMenu *menu)
 {
-  g_source_remove (menu->priv->scroll_id);
-  menu->priv->scroll_id = 0;
-}
-
-static void 
-hildon_desktop_popup_menu_scroll_cb (GtkWidget *widget, HildonDesktopPopupMenu *menu)
-{
-  if (widget == menu->priv->scroll_up)
-    menu->priv->scroll_dir = -1;
-  else
-    menu->priv->scroll_dir = 1;
-
-  hildon_desktop_popup_menu_scroll_step (menu);
+  hildon_desktop_popup_menu_remove_scroll_timeout (menu);
 }
 
 static gboolean 
@@ -694,7 +713,8 @@ hildon_desktop_popup_menu_press_event (GtkWidget      *widget,
     {
       if (GTK_IS_MENU_ITEM (l->data))
       {
-        if (menu->priv->selected_item != GTK_MENU_ITEM (l->data))
+        if (hildon_desktop_popup_menu_item_is_selectable (GTK_WIDGET (l->data)) &&
+            menu->priv->selected_item != GTK_MENU_ITEM (l->data))
         {
           gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
           gtk_item_select (GTK_ITEM (l->data));
@@ -845,6 +865,36 @@ hildon_desktop_popup_menu_parent_size (HildonDesktopPopupMenu *menu)
     }
   }
 
+  if (menu->priv->resize_parent && GTK_IS_WINDOW (menu->priv->parent))
+  {
+    gint border_width;
+
+    g_object_get (G_OBJECT (menu->priv->parent),
+		  "border-width", &border_width,
+		  NULL); 
+    
+    gtk_widget_set_size_request (menu->priv->parent,
+		    		 parent_req.width,
+				 show_scroll_controls ? 
+				 max_height : 
+				 d_height + (2 * border_width));
+
+    if (GTK_WIDGET_REALIZED (menu->priv->parent))
+      gdk_window_resize (menu->priv->parent->window,
+		         parent_req.width,
+			 show_scroll_controls ? 
+			 max_height : 
+	                 d_height + (2 * border_width));
+
+    if (GTK_WIDGET_MAPPED (menu->priv->parent))
+    {
+      g_signal_emit_by_name (menu, "popup-menu-resize");
+
+      gtk_widget_queue_resize (GTK_WIDGET (menu)); 
+      gtk_widget_queue_resize (menu->priv->parent);
+    }
+  }
+
   if (show_scroll_controls)
   {
     hildon_desktop_popup_menu_show_controls (menu);	  
@@ -860,36 +910,6 @@ hildon_desktop_popup_menu_parent_size (HildonDesktopPopupMenu *menu)
 
   gtk_widget_queue_resize (menu->priv->viewport);
  
-  if (menu->priv->resize_parent && GTK_IS_WINDOW (menu->priv->parent))
-  {
-    gint border_width;
-
-    g_object_get (G_OBJECT (menu->priv->parent),
-		  "border-width", &border_width,
-		  NULL); 
-    
-    gtk_widget_set_size_request (menu->priv->parent,
-		    		 parent_req.width,
-				 show_scroll_controls ? 
-				 max_height : 
-				 d_height + (2 * border_width));
-
-    if (GTK_WIDGET_MAPPED (menu->priv->parent))
-    {	    
-      g_signal_emit_by_name (menu, "popup-menu-resize");
-	    
-      gtk_widget_queue_resize (GTK_WIDGET (menu));    
-      gtk_widget_queue_resize (menu->priv->parent);
-    }
-    
-    if (GTK_WIDGET_REALIZED (menu->priv->parent))
-      gdk_window_resize (menu->priv->parent->window,
-		         parent_req.width,
-			 show_scroll_controls ? 
-			 max_height : 
-	                 d_height + (2 * border_width));
-  }
-
   g_list_free (children);
 }	
 
@@ -939,21 +959,11 @@ hildon_desktop_popup_menu_select_next_prev_item (HildonDesktopPopupMenu *menu, g
   }	  
 }
 
-void
-hildon_desktop_popup_menu_add_item (HildonDesktopPopupMenu *menu, GtkMenuItem *item)
+static void 
+hildon_desktop_popup_menu_real_add_item (HildonDesktopPopupMenu *menu, 
+				         GtkMenuItem *item, 
+					 gint item_width)
 {
-  GtkRequisition req;
-  gint item_width = -1;
-  
-  g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu));
-  g_return_if_fail (GTK_IS_MENU_ITEM (item));
-
-  if (GTK_IS_WINDOW (menu->priv->parent))
-  {
-      gtk_widget_size_request (menu->priv->parent, &req);
-      item_width = req.width;
-  }
-  
   gtk_box_pack_end (GTK_BOX (menu->priv->box_items),
 		    GTK_WIDGET (item),
 		    FALSE, FALSE, 0);
@@ -973,11 +983,81 @@ hildon_desktop_popup_menu_add_item (HildonDesktopPopupMenu *menu, GtkMenuItem *i
   
   gtk_widget_show (GTK_WIDGET (item));
 
+  menu->priv->n_items++;
+}
+
+static void
+hildon_desktop_popup_menu_real_remove_item (HildonDesktopPopupMenu *menu, GtkMenuItem *item)
+{
+  if (menu->priv->selected_item == item)
+    hildon_desktop_popup_menu_deselect_item (menu, item);
+
+  gtk_container_remove (GTK_CONTAINER (menu->priv->box_items), GTK_WIDGET (item));
+ 
+  menu->priv->n_items--;
+}
+
+void
+hildon_desktop_popup_menu_add_item (HildonDesktopPopupMenu *menu, GtkMenuItem *item)
+{
+  GtkRequisition req;
+  gint item_width = -1;
+  
+  g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu));
+  g_return_if_fail (GTK_IS_MENU_ITEM (item));
+
+  if (GTK_IS_WINDOW (menu->priv->parent))
+  {
+      gtk_widget_size_request (menu->priv->parent, &req);
+      item_width = req.width;
+  }
+
+  hildon_desktop_popup_menu_real_add_item (menu, item, item_width);
+  
   gtk_widget_set_size_request (menu->priv->box_buttons, 
 		  	       item_width, 
 			       menu->priv->item_height + 2);
 
-  menu->priv->n_items++;
+  hildon_desktop_popup_menu_parent_size (menu);
+}
+
+void
+hildon_desktop_popup_menu_replace_items (HildonDesktopPopupMenu *menu, GList *items)
+{
+  GtkRequisition req;
+  GList *children, *l;
+  gint item_width = -1;
+
+  g_assert (HILDON_DESKTOP_IS_POPUP_MENU (menu));
+
+  if (GTK_IS_WINDOW (menu->priv->parent))
+  {
+      gtk_widget_size_request (menu->priv->parent, &req);
+      item_width = req.width;
+  }
+
+  children = hildon_desktop_popup_menu_get_children (menu);
+ 
+  for (l = children; l != NULL; l = l->next)
+  {
+    GtkWidget *item = GTK_WIDGET (l->data);
+    
+    hildon_desktop_popup_menu_real_remove_item (menu, GTK_MENU_ITEM (item));
+  }
+ 
+  for (l = items; l != NULL; l = l->next)
+  {
+    GtkWidget *item = GTK_WIDGET (l->data);
+
+    if (GTK_IS_MENU_ITEM (item))
+      hildon_desktop_popup_menu_real_add_item (menu, 
+		                               GTK_MENU_ITEM (item), 
+				   	       item_width);
+  }
+  
+  gtk_widget_set_size_request (menu->priv->box_buttons, 
+		  	       item_width, 
+			       menu->priv->item_height + 2);
 
   hildon_desktop_popup_menu_parent_size (menu);
 }
@@ -988,20 +1068,19 @@ hildon_desktop_popup_menu_remove_item (HildonDesktopPopupMenu *menu, GtkMenuItem
   g_assert (HILDON_DESKTOP_IS_POPUP_MENU (menu));
   g_return_if_fail (GTK_IS_MENU_ITEM (item));
 
-  gtk_container_remove (GTK_CONTAINER (menu->priv->box_items), GTK_WIDGET (item));
- 
-  menu->priv->n_items--;
-
+  hildon_desktop_popup_menu_real_remove_item (menu, item);
+  
   hildon_desktop_popup_menu_parent_size (menu);
+
 }
 
 GList *
 hildon_desktop_popup_menu_get_children (HildonDesktopPopupMenu *menu)
 {
   g_return_val_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu), NULL);
-	
+
   GList *list = gtk_container_get_children (GTK_CONTAINER (menu->priv->box_items));
-	
+
   return list;
 }
 
@@ -1011,12 +1090,15 @@ hildon_desktop_popup_menu_select_item (HildonDesktopPopupMenu *menu, GtkMenuItem
   g_return_if_fail (HILDON_DESKTOP_IS_POPUP_MENU (menu));
   g_return_if_fail (GTK_IS_MENU_ITEM (item));
 
-  if (GTK_IS_ITEM (menu->priv->selected_item))
-    gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));	    
+  if (hildon_desktop_popup_menu_item_is_selectable (GTK_WIDGET (item)))
+  {
+    if (GTK_IS_ITEM (menu->priv->selected_item))
+      gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));	    
 
-  gtk_item_select (GTK_ITEM (item));
-  menu->priv->selected_item = item;
-  hildon_desktop_menu_check_scroll_item (menu);
+    gtk_item_select (GTK_ITEM (item));
+    menu->priv->selected_item = item;
+    hildon_desktop_menu_check_scroll_item (menu);
+  }
 } 
 
 void 
@@ -1042,7 +1124,7 @@ hildon_desktop_popup_menu_select_first_item (HildonDesktopPopupMenu *menu)
 
   for (l = children; l != NULL; l = g_list_next (l))
   {	  
-    if (!GTK_IS_SEPARATOR_MENU_ITEM (l->data))
+    if (hildon_desktop_popup_menu_item_is_selectable (GTK_WIDGET (l->data)))
     {
       if (GTK_IS_ITEM (menu->priv->selected_item))
         gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
@@ -1067,7 +1149,7 @@ hildon_desktop_popup_menu_select_last_item (HildonDesktopPopupMenu *menu)
 
   for (l = g_list_last (children); l != NULL; l = g_list_previous (l))
   {
-    if (!GTK_IS_SEPARATOR_MENU_ITEM (l->data))
+    if (hildon_desktop_popup_menu_item_is_selectable (GTK_WIDGET (l->data)))
     {
       if (GTK_IS_ITEM (menu->priv->selected_item))
         gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
