@@ -1603,6 +1603,7 @@ remove_widget (GtkWidget *widget, GList *region)
   GdkRectangle r = {0};
   gint x, y;
   gint padding;
+  guint width, height;
 
   gtk_container_child_get (GTK_CONTAINER (widget->parent), widget,
                            "x", &x,
@@ -1613,17 +1614,31 @@ remove_widget (GtkWidget *widget, GList *region)
                 "applet-padding", &padding,
                 NULL);
 
+  if (widget->allocation.width == 1 && widget->allocation.height == 1)
+  {
+    GtkRequisition req;
+
+    gtk_widget_size_request (widget, &req);
+    width = req.width;
+    height = req.height;
+  }
+  else
+  {
+    width = widget->allocation.width;
+    height = widget->allocation.height;
+  }
+
   r.x = x?x-padding:0;
   r.y = y?y-padding:0;
 
-  r.width = widget->allocation.width;
-  if (x + widget->allocation.width != widget->parent->allocation.width)
+  r.width = width;
+  if (x + width != widget->parent->allocation.width)
     r.width += padding;
   if (x)
     r.width += padding;
 
-  r.height = widget->allocation.height;
-  if (y + widget->allocation.height != widget->parent->allocation.height)
+  r.height = height;
+  if (y + height != widget->parent->allocation.height)
     r.height += padding;
   if (y)
     r.height += padding;
@@ -1670,8 +1685,9 @@ hildon_home_area_place (HildonHomeArea *area, GtkWidget *w)
 
   region = g_list_sort (region, (GCompareFunc)sort_rectangles);
 
+  gtk_widget_set_child_visible (w, FALSE);
+  gtk_widget_set_parent (w, GTK_WIDGET (area));
   gtk_widget_size_request (w, &req);
-  g_debug ("size request: %ix%i", req.width, req.height);
 
   if (req.width > area_rectangle->width ||
       req.height > area_rectangle->height)
@@ -1688,6 +1704,7 @@ hildon_home_area_place (HildonHomeArea *area, GtkWidget *w)
     g_hash_table_insert (priv->layout, g_strdup (name), layout);
     hildon_home_area_put (area, w, 0, 0, G_MAXINT);
     g_signal_emit_by_name (area, "layout-changed");
+    gtk_widget_set_child_visible (w, TRUE);
 
     g_list_foreach (region, (GFunc)g_free, NULL);
     g_list_free (region);
@@ -1716,6 +1733,7 @@ hildon_home_area_place (HildonHomeArea *area, GtkWidget *w)
 
         g_hash_table_insert (priv->layout, g_strdup (name), layout);
         hildon_home_area_put (area, w, r->x, r->y, G_MAXINT);
+        gtk_widget_set_child_visible (w, TRUE);
         g_signal_emit_by_name (area, "layout-changed");
 
 
@@ -1787,6 +1805,8 @@ hildon_home_area_remove (GtkContainer *area, GtkWidget *applet)
 
   }
 
+  gtk_widget_unparent (applet);
+
 }
 
 /* Public functions */
@@ -2083,7 +2103,8 @@ hildon_home_area_put (HildonHomeArea *area,
 
   g_return_if_fail (HILDON_IS_HOME_AREA (area) && GTK_IS_WIDGET (widget));
 
-  gtk_widget_set_parent (widget, GTK_WIDGET (area));
+  if (widget->parent != GTK_WIDGET (area))
+    gtk_widget_set_parent (widget, GTK_WIDGET (area));
 
   if (HILDON_DESKTOP_IS_HOME_ITEM (widget))
     g_object_get (widget,
