@@ -279,7 +279,8 @@ GList *get_desktop_files(gchar *directory, GList *desktop_files)
 				g_str_has_suffix(current_path,
 					DESKTOP_FILE_SUFFIX)) {
 
-            gchar *type = NULL;
+                        gchar *type = NULL;
+                        gboolean no_display;
 
 			key_file = g_key_file_new();
 
@@ -299,6 +300,22 @@ GList *get_desktop_files(gchar *directory, GList *desktop_files)
 			}
 						
 
+                        /* Check for the optional NoDisplay key.
+                         * If found and true, continue, we won't
+                         * have it in the menu */
+                        no_display = g_key_file_get_boolean(key_file,
+                                                            DESKTOP_ENTRY_GROUP,
+                                                            DESKTOP_ENTRY_NO_DISPLAY_FIELD,
+                                                            &error);
+
+                        if (error)
+                          g_clear_error (&error);
+                        else if (no_display)
+                        {
+                          g_free (current_path);
+                          continue;
+                        }
+
 			type = g_key_file_get_string(
                                          key_file,
                                          DESKTOP_ENTRY_GROUP,
