@@ -472,10 +472,22 @@ hd_keys_shortcut_new (HDKeysConfig *keys,
     else
       keydef[0] = g_ascii_tolower(keydef[0]);
 
-    if ((ks = XStringToKeysym(keydef)) == (KeySym)NULL)
-      return NULL; /* Couldn't find a keysym */
+    switch (*keydef)
+    {
+      case '-':
+ 	ks = XK_minus;	    
+	break;
+	
+      case '+':
+        ks = XK_plus;
+	break;
+	
+      default:
+        if ((ks = XStringToKeysym(keydef)) == (KeySym)NULL)
+          return NULL; /* Couldn't find a keysym */
+    }
   }
-
+  
   /* Cannot install grab for keys that we are supposed to fake */
   switch (ks)
   {
