@@ -320,6 +320,9 @@ hd_app_menu_tree_constructor (GType                   type,
   gtk_paned_add2 (GTK_PANED (object), priv->content_scrolled_window);
   g_object_ref (priv->content_scrolled_window);
 
+  /* FIXME: somehow set dynamically */
+  gtk_paned_set_position (GTK_PANED (object), 300);
+
   return object;
 
 }
@@ -922,6 +925,7 @@ hd_app_menu_tree_set_model (HDAppMenuTree *tree, GtkTreeModel *model)
   }
 
   g_object_notify (G_OBJECT (tree), "model");
+
 }
 
 gboolean
