@@ -370,7 +370,7 @@ hd_applications_menu_activate_category (GtkMenuItem *item, HDApplicationsMenu *b
   static GtkMenuItem *last_selected_item = NULL;
 
   GList *sub_items, *i;
-  
+
   sub_items = hildon_desktop_popup_menu_get_children 
 	  (button->priv->menu_applications);
   
@@ -383,26 +383,22 @@ hd_applications_menu_activate_category (GtkMenuItem *item, HDApplicationsMenu *b
       GtkMenuItem *child = (GtkMenuItem *) i->data;
 
       g_object_ref (child);
-
-      hildon_desktop_popup_menu_remove_item
-        (button->priv->menu_applications, child);
     }
 
     g_list_free (sub_items);
 
     sub_items = (GList *) g_object_get_data (G_OBJECT (item),
-          	                           CATEGORY_SUB_ITEMS);
+          	                             CATEGORY_SUB_ITEMS);
 
-    for (i = g_list_first (sub_items); i; i = i->next)
-    {
-      GtkMenuItem *child = (GtkMenuItem *) i->data;
-
-      hildon_desktop_popup_menu_add_item
-        (button->priv->menu_applications, child);
-    }
+    hildon_desktop_popup_menu_replace_items (button->priv->menu_applications, 
+		    		  	     sub_items);
+  }
+  else
+  {
+    button->priv->focus_applications = TRUE;
   }
 
-  if (button->priv->focus_applications &&
+  if (button->priv->focus_applications && sub_items != NULL &&
       GTK_WIDGET_IS_SENSITIVE (sub_items->data))
   {
     hildon_desktop_popup_menu_select_first_item (button->priv->menu_applications);
@@ -751,7 +747,7 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
       if (!first_item)
       {
         GtkWidget *separator = gtk_separator_menu_item_new ();
-	
+
         if (level > 0)
           gtk_widget_set_name (GTK_WIDGET (separator), HD_APPS_MENU_APP_MENU_ITEM_NAME);
         else
@@ -850,6 +846,10 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
   
   g_return_if_fail (button);
 
+  g_signal_handlers_block_by_func (button->priv->button,
+  				   hd_applications_menu_button_toggled, 
+  				   button);
+
   box = gtk_hbox_new (TRUE, 0);
   
   popup_window =
@@ -975,6 +975,10 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
 
   /* Now populate the menu */
   hd_applications_menu_populate (button);
+
+  g_signal_handlers_unblock_by_func (button->priv->button,
+  				     hd_applications_menu_button_toggled, 
+  				     button);
 }
 
 static void
