@@ -409,7 +409,7 @@ hd_desktop_plugin_list_to_conf (GList *plugin_list, const gchar *config_file)
 
   keyfile = g_key_file_new ();
 
-  for (iter = plugin_list; iter; iter = g_list_next (iter))
+  for (iter = g_list_last (plugin_list); iter; iter = g_list_previous (iter))
   {
     g_key_file_set_string (keyfile,
                            (gchar *) iter->data,
@@ -1569,7 +1569,7 @@ hd_desktop_system_notifications_filter (GtkTreeModel *model,
   {
     return FALSE;
   }
-  else if (g_str_has_prefix (g_value_get_string (category), "system."))
+  else if (g_str_has_prefix (g_value_get_string (category), "system"))
   {
     return TRUE;
   } 
