@@ -561,6 +561,37 @@ pid_from_exec_name (char *exec_name)
   return the_pid;
 }
 
+static void
+hd_home_window_save_background (HDHomeWindow *window)
+{
+  HDHomeWindowPrivate  *priv;
+  gchar                *conffile;
+  GError               *save_error = NULL;
+
+  priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  conffile = g_build_path (G_DIR_SEPARATOR_S,
+                           g_get_home_dir (),
+                           HD_DESKTOP_USER_PATH,
+                           HD_HOME_BACKGROUND_CONF_FILE,
+                           NULL);
+
+  hildon_desktop_background_save (priv->background,
+                                  conffile,
+                                  &save_error);
+
+  if (save_error)
+    {
+      g_warning ("Error when saving background settings to %s: %s",
+                 conffile,
+                 save_error->message);
+      /* Assume flash full */
+      hd_home_window_show_information_note (window, HH_FLASH_FULL_TEXT);
+      g_error_free (save_error);
+    }
+
+  g_free (conffile);
+}
+
 static void
 background_apply_callback (HildonDesktopBackground     *background,
                            Picture                      picture,
@@ -665,8 +696,6 @@ background_apply_and_save_callback (HildonDesktopBackground    *background,
                                     HDHomeWindow               *window)
 {
   HDHomeWindowPrivate  *priv;
-  gchar                *conffile;
-  GError               *save_error = NULL;
 
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
 
@@ -676,27 +705,7 @@ background_apply_and_save_callback (HildonDesktopBackground    *background,
   if (error)
     return;
 
-  conffile = g_build_path (G_DIR_SEPARATOR_S,
-                           g_get_home_dir (),
-                           HD_DESKTOP_USER_PATH,
-                           HD_HOME_BACKGROUND_CONF_FILE,
-                           NULL);
-
-  hildon_desktop_background_save (priv->background,
-                                  conffile,
-                                  &save_error);
-
-  if (save_error)
-    {
-      g_warning ("Error when saving background settings to %s: %s",
-                 conffile,
-                 save_error->message);
-      /* Assume flash full */
-      hd_home_window_show_information_note (window, HH_FLASH_FULL_TEXT);
-      g_error_free (save_error);
-    }
-
-  g_free (conffile);
+  hd_home_window_save_background (window);
 
 }
 
@@ -1520,6 +1529,9 @@ hd_home_window_set_background_reponse (HDHomeWindow *window,
                    (HildonDesktopBackgroundApplyCallback)background_apply_and_save_callback,
                    window);
             }
+          else
+            hd_home_window_save_background (window);
+
           g_object_unref (priv->previous_background);
           break;
       case HILDON_HOME_SET_BG_RESPONSE_PREVIEW:
