@@ -142,6 +142,26 @@ hd_panel_window_dialog_notify_condition (GObject *object,
 					 HDPanelWindowDialog *window)
 {
   g_signal_emit_by_name (window, "update-status", HILDON_DESKTOP_ITEM (object)->id, condition);
+
+  /* Some day instropection will work for GObject signals and this code won't make any sense */
+
+  DBusMessage *message;
+
+  message =
+    dbus_message_new_signal ("/org/hildon/Statusbar",
+                             "org.hildon.Statusbar",
+                             "UpdateStatus");
+
+  dbus_message_append_args (message,
+                            DBUS_TYPE_STRING, &(HILDON_DESKTOP_ITEM (object)->id),
+                           DBUS_TYPE_BOOLEAN, condition,
+                           DBUS_TYPE_INVALID);
+
+  dbus_connection_send (dbus_g_connection_get_connection(window->priv->connection),
+                       message,
+                       NULL);
+
+  dbus_message_unref (message);
 }
 
 static void
