@@ -58,6 +58,8 @@
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 
+#include <hildon/hildon-helper.h>
+
 /* Menu item strings */
 #define AS_HOME_ITEM 		_("tana_fi_home")
 #define AS_HOME_ITEM_ICON 	"qgn_list_home"
@@ -83,6 +85,10 @@
 #define AS_SEPARATOR_HEIGHT        10
 #define AS_BUTTON_BOX_PADDING      10
 
+
+#define AS_TEXT_COLOR              "PluginMenuTextColor"
+#define AS_SECONDARY_TEXT_COLOR    "PluginMenuSecondaryTextColor"
+
 /*
  * HDSwitcherMenuItem
  */
@@ -336,14 +342,22 @@ hd_switcher_menu_item_constructor (GType                  type,
   gtk_widget_show (vbox);
       
   priv->label2 = gtk_label_new (NULL);
+  hildon_helper_set_logical_color (priv->label2,
+                                   GTK_RC_FG,
+                                   GTK_STATE_NORMAL,
+                                   AS_SECONDARY_TEXT_COLOR);
   gtk_widget_set_composite_name (GTK_WIDGET (priv->label2), "as-app-menu-label-desc");
-  gtk_misc_set_alignment (GTK_MISC (priv->label2), 0.0, 0.5);
+  gtk_misc_set_alignment (GTK_MISC (priv->label2), 0.0, 0.1);
   gtk_box_pack_end (GTK_BOX (vbox), priv->label2, TRUE, TRUE, 0);
   gtk_widget_show (priv->label2);
   
   priv->label = gtk_label_new (NULL);
+  hildon_helper_set_logical_color (priv->label,
+                                   GTK_RC_FG,
+                                   GTK_STATE_NORMAL,
+                                   AS_TEXT_COLOR);
   gtk_widget_set_composite_name (GTK_WIDGET (priv->label), "as-app-menu-label");
-  gtk_misc_set_alignment (GTK_MISC (priv->label), 0.0, 0.5);
+  gtk_misc_set_alignment (GTK_MISC (priv->label), 0.0, 0.9);
   gtk_box_pack_end (GTK_BOX (vbox), priv->label, TRUE, TRUE, 0);
   gtk_widget_show (priv->label);
 
