@@ -198,7 +198,7 @@ static void hd_switcher_menu_toggled_cb (GtkWidget *button, HDSwitcherMenu *swit
 static void hd_switcher_menu_scroll_to (HildonDesktopPopupWindow *window,
 					HDSwitcherMenu *switcher);
 
-static void hd_switcher_menu_add_clear_notifications_button (HDSwitcherMenu *switcher);
+static void hd_switcher_menu_create_clear_notifications_button (HDSwitcherMenu *switcher);
 
 static void hd_switcher_menu_refresh_application_items (HDSwitcherMenu *switcher);
 static void hd_switcher_menu_refresh_notification_items (HDSwitcherMenu *switcher);
@@ -314,29 +314,32 @@ hd_switcher_menu_create_separator (gboolean is_application_menu)
 }
 
 static GdkPixbuf *
-hd_switcher_menu_get_icon_from_theme (HDSwitcherMenu *switcher,
-                                     const gchar     *icon_name,
-                                     gint             size)
+hd_switcher_menu_get_icon_from_theme (const gchar     *icon_name,
+                                      gint             size)
 {
+  static GtkIconTheme *icon_theme = NULL;
+	
   GError *error;
   GdkPixbuf *retval;
 
+  if (!icon_theme)
+    icon_theme = gtk_icon_theme_get_default ();
+
   if (!icon_name)
     return NULL;
 
-  switcher->priv->icon_theme = gtk_icon_theme_get_default ();
-
-  g_return_val_if_fail (switcher->priv->icon_theme, NULL);
+  g_return_val_if_fail (icon_theme, NULL);
 
   if (!icon_name || icon_name[0] == '\0')
     return NULL;
 
   error = NULL;
-  retval = gtk_icon_theme_load_icon (switcher->priv->icon_theme,
+  retval = gtk_icon_theme_load_icon (icon_theme,
                                      icon_name,
                                      size == -1 ? AS_ICON_SIZE : size,
                                      0,
                                      &error);
+
   if (error)
   {
     g_warning ("Could not load icon '%s': %s\n",
@@ -414,15 +417,23 @@ hd_switcher_menu_popup_window_keypress_cb (GtkWidget      *widget,
        (switcher->priv->menu_notifications);
 
     if (notifications)    
-    {	  
+    {
+      GtkMenuItem *selected_item;
+      GList *last = g_list_last (notifications);
+	    
       hildon_desktop_popup_window_jump_to_pane 
         (switcher->priv->popup_window, 0);	    
 
-      hildon_desktop_popup_menu_select_item
-        (switcher->priv->menu_applications, NULL);
+      selected_item =
+        hildon_desktop_popup_menu_get_selected_item (
+ 	  	                     switcher->priv->menu_applications);
+     
+      hildon_desktop_popup_menu_deselect_item
+        (switcher->priv->menu_applications, selected_item);
 
-      hildon_desktop_popup_menu_select_first_item 
-        (switcher->priv->menu_notifications);	    
+      hildon_desktop_popup_menu_select_item 
+        (switcher->priv->menu_notifications,
+	 GTK_MENU_ITEM (last->data));
 
       g_list_free (notifications);
       
@@ -457,14 +468,27 @@ hd_switcher_menu_popup_window_pane_keypress_cb (GtkWidget      *widget,
 
   if (cancel_menu)
   {
+    GtkMenuItem *selected_item;
+    GList *applications, *last;
+
+    applications = 
+      hildon_desktop_popup_menu_get_children (switcher->priv->menu_applications);
+
+    last = g_list_last (applications);
+    
     hildon_desktop_popup_window_jump_to_pane
       (switcher->priv->popup_window, -1);
 
-    hildon_desktop_popup_menu_select_item
-      (switcher->priv->menu_notifications, NULL);
+    selected_item =
+      hildon_desktop_popup_menu_get_selected_item (
+		                   switcher->priv->menu_notifications);
+    
+    hildon_desktop_popup_menu_deselect_item
+      (switcher->priv->menu_notifications, selected_item);
 
-    hildon_desktop_popup_menu_select_first_item
-      (switcher->priv->menu_applications);
+    hildon_desktop_popup_menu_select_item
+      (switcher->priv->menu_applications,
+       GTK_MENU_ITEM (last->data));
 
     return TRUE;
   } 
@@ -666,10 +690,10 @@ hd_switcher_menu_add_notification_group (gpointer key,
 					 gpointer user_data)
 {
   HDSwitcherMenuNotificationGroup *ngroup;
-  HDSwitcherMenu *switcher;
+  GList **items;
   
   ngroup = (HDSwitcherMenuNotificationGroup *) value;
-  switcher = (HDSwitcherMenu *) user_data;	
+  items = (GList **) user_data;	
  
   if (ngroup->is_active)
   {
@@ -680,11 +704,9 @@ hd_switcher_menu_add_notification_group (gpointer key,
     summary = g_strdup_printf (_(ngroup->label), 
 		               g_list_length (ngroup->notifications));
     
-    icon = hd_switcher_menu_get_icon_from_theme (switcher, ngroup->icon, -1);
-    
-    hildon_desktop_popup_menu_add_item
-      (switcher->priv->menu_notifications,
-      GTK_MENU_ITEM (hd_switcher_menu_create_separator (FALSE)));
+    icon = hd_switcher_menu_get_icon_from_theme (ngroup->icon, -1);
+
+    *items = g_list_append (*items, hd_switcher_menu_create_separator (FALSE));
 
     menu_item = hd_switcher_menu_item_new_from_notification_group
       (ngroup->notifications, icon, summary, ngroup->dbus_callback, TRUE);
@@ -692,17 +714,16 @@ hd_switcher_menu_add_notification_group (gpointer key,
     gtk_widget_set_name (GTK_WIDGET (menu_item), 
       		         HD_SWITCHER_MENU_NOT_MENU_ITEM_NAME);
 
-    hildon_desktop_popup_menu_add_item
-     (switcher->priv->menu_notifications, GTK_MENU_ITEM (menu_item));
+    *items = g_list_append (*items, menu_item);
   }
 }
 
 static void
-hd_switcher_menu_add_notification_groups (HDSwitcherMenu *switcher)
+hd_switcher_menu_add_notification_groups (HDSwitcherMenu *switcher, GList **items)
 {
   g_hash_table_foreach (switcher->priv->notification_groups, 
 		        hd_switcher_menu_add_notification_group,
-			switcher);
+			items);
 }
 
 static void
@@ -826,17 +847,26 @@ hd_switcher_menu_load_notification_groups (HDSwitcherMenu *switcher)
 }
 
 #ifdef HAVE_LIBOSSO
-static
-void hd_switcher_menu_hw_cb (osso_hw_state_t *state, gpointer data)
+static void 
+hd_switcher_menu_display_state_cb (osso_display_state_t state, gpointer data)
 {
-  g_return_if_fail (state && data);
+  gboolean system_inactivity;
+	
+  g_return_if_fail (data);
 
   HDSwitcherMenu *switcher = HD_SWITCHER_MENU (data);
   HDSwitcherMenuPrivate *priv = switcher->priv;
 
-  if (state->system_inactivity_ind != priv->system_inactivity)
+  if (state == OSSO_DISPLAY_DIMMED)
+    return;
+
+  system_inactivity = (state == OSSO_DISPLAY_OFF); 
+
+  g_debug ("LALALALALALALALA: %d", system_inactivity);
+  
+  if (system_inactivity != priv->system_inactivity)
   {
-    priv->system_inactivity = state->system_inactivity_ind;
+    priv->system_inactivity = system_inactivity;
 
     hd_switcher_menu_refresh_application_items (switcher);
     hd_switcher_menu_refresh_notification_items (switcher);
@@ -852,7 +882,6 @@ static void
 hd_switcher_menu_osso_init (HDSwitcherMenu *switcher)
 {
   HDSwitcherMenuPrivate *priv = switcher->priv;
-  osso_hw_state_t hs = {0};
   
   priv->osso = osso_initialize ("SM_DIMMED_infoprint", "0.1", FALSE, NULL);
 
@@ -864,12 +893,9 @@ hd_switcher_menu_osso_init (HDSwitcherMenu *switcher)
   }
 
   /* Register stystem inactivity handler */
-  hs.system_inactivity_ind = TRUE;
-
-  osso_hw_set_event_cb (priv->osso, 
-		        &hs,
-                        hd_switcher_menu_hw_cb, 
-		        switcher);
+  osso_hw_set_display_event_cb (priv->osso,
+                                hd_switcher_menu_display_state_cb,
+                                switcher);
 }
 #endif
 
@@ -900,7 +926,7 @@ hd_switcher_menu_constructor (GType gtype,
   /*gtk_widget_set_size_request (button, -1, AS_MENU_BUTTON_HEIGHT);*/
   gtk_widget_set_size_request (GTK_WIDGET (switcher), -1, AS_MENU_BUTTON_HEIGHT);
 
-  pixbuf = hd_switcher_menu_get_icon_from_theme (switcher, AS_MENU_BUTTON_ICON, -1);
+  pixbuf = hd_switcher_menu_get_icon_from_theme (AS_MENU_BUTTON_ICON, -1);
   switcher->priv->icon = gtk_image_new_from_pixbuf (pixbuf);
   gtk_container_add (GTK_CONTAINER (button), switcher->priv->icon);
   gtk_widget_show (switcher->priv->icon);
@@ -1169,8 +1195,8 @@ hd_switcher_create_external_window (HDSwitcherMenu *switcher)
   switcher->priv->window_dialog =
     GTK_WIDGET (g_object_new (HILDON_DESKTOP_TYPE_PANEL_WINDOW_DIALOG,
                               "x", 0,
-                              "y", gdk_screen_height () - AS_BUTTON_HEIGHT * 2,
-                              "width", GTK_WIDGET (switcher)->allocation.width,
+                              "y", gdk_screen_height () - GTK_WIDGET (switcher)->allocation.height,
+                              "width", GTK_WIDGET (switcher)->allocation.height,
                               "height", AS_BUTTON_HEIGHT*2,
                               "move", TRUE,
                               "use-old-titlebar", FALSE,
@@ -1276,7 +1302,7 @@ hd_switcher_menu_clear_item_activated (GtkMenuItem *menuitem, HDSwitcherMenu *sw
 }
 	
 static void 
-hd_switcher_menu_add_clear_notifications_button (HDSwitcherMenu *switcher)
+hd_switcher_menu_create_clear_notifications_button (HDSwitcherMenu *switcher)
 {
   GtkWidget *hbox, *image, *label;
   GdkPixbuf *icon;
@@ -1285,9 +1311,7 @@ hd_switcher_menu_add_clear_notifications_button (HDSwitcherMenu *switcher)
     
   hbox = gtk_hbox_new (FALSE, 0);
 
-  icon = hd_switcher_menu_get_icon_from_theme (switcher,
-        	  			         AS_CLEAR_ITEM_ICON, 
-      					 -1);
+  icon = hd_switcher_menu_get_icon_from_theme (AS_CLEAR_ITEM_ICON, -1);
   
   image = gtk_image_new_from_pixbuf (icon);
   
@@ -1317,9 +1341,8 @@ hd_switcher_menu_add_clear_notifications_button (HDSwitcherMenu *switcher)
                           G_CALLBACK (hd_switcher_menu_clear_item_activated),
                           switcher);
 
-  hildon_desktop_popup_menu_add_item
-   (switcher->priv->menu_notifications, 
-    GTK_MENU_ITEM (switcher->priv->clear_events_menu_item));
+  gtk_widget_set_name (GTK_WIDGET (switcher->priv->clear_events_menu_item), 
+		       HD_SWITCHER_MENU_NOT_MENU_ITEM_NAME);
 }
 
 static void
@@ -1661,15 +1684,13 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
 {
   GtkWidget *menu_item = NULL, *first_item = NULL;
   GtkTreeModel *nm = GTK_TREE_MODEL (switcher->nm);
-  GList *children = NULL, *l;
+  GList *children = NULL;
   GtkTreeIter  iter;	
 
-  children =
-    hildon_desktop_popup_menu_get_children (switcher->priv->menu_notifications);
-
   if (gtk_tree_model_iter_n_children (nm, NULL) >= AS_ITEMS_LIMIT)
   {
-    hd_switcher_menu_add_clear_notifications_button (switcher);
+    hd_switcher_menu_create_clear_notifications_button (switcher);
+    children = g_list_append (children, switcher->priv->clear_events_menu_item);
     first_item = switcher->priv->clear_events_menu_item;
   }
   else
@@ -1681,21 +1702,13 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
     }
   }
   
-  for (l = children; l != NULL; l = g_list_next (l))
-  {
-    if (l->data != switcher->priv->clear_events_menu_item)
-      gtk_widget_destroy (GTK_WIDGET (l->data));
-  }
-
-  g_list_free (children);
-
   if (gtk_tree_model_get_iter_first (nm, &iter))
   {
     GdkPixbuf *icon;
     HDSwitcherMenuNotificationGroup *ngroup;
     GHashTable *hints;
     GValue *hint;
-    gchar *summary, *body;
+    gchar *summary, *body, *icon_name;
     gboolean ack, icon_is_set = FALSE;
     gint id;
 
@@ -1729,7 +1742,7 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
       }
       while (gtk_tree_model_iter_next (nm, &iter));
 
-      hd_switcher_menu_add_notification_groups (switcher);
+      hd_switcher_menu_add_notification_groups (switcher, &children);
 
       gtk_tree_model_get_iter_first (nm, &iter);
     }
@@ -1742,6 +1755,7 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
                           &iter,
                           HD_NM_COL_ID, &id,
                           HD_NM_COL_ICON, &icon,
+                          HD_NM_COL_ICON_NAME, &icon_name,
                           HD_NM_COL_SUMMARY, &summary,
                           HD_NM_COL_BODY, &body,
                           HD_NM_COL_HINTS, &hints,
@@ -1758,9 +1772,8 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
       if (ngroup == NULL || !ngroup->is_active)
       {
         if (first_item)
-          hildon_desktop_popup_menu_add_item
-            (switcher->priv->menu_notifications,
-            GTK_MENU_ITEM (hd_switcher_menu_create_separator (FALSE)));
+	  children = g_list_append (children, 
+			            hd_switcher_menu_create_separator (FALSE));
 
         menu_item =
           hd_switcher_menu_item_new_from_notification
@@ -1777,24 +1790,33 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
           		        G_CALLBACK (hd_switcher_menu_item_activated),
           		        switcher);
 
-        hildon_desktop_popup_menu_add_item
-         (switcher->priv->menu_notifications, GTK_MENU_ITEM (menu_item));
+	children = g_list_append (children, menu_item);
 
         if (!ack && !icon_is_set)
         {
 	  switcher->priv->blinking_notification = id;
 
-          hd_switcher_menu_replace_blinking_icon (switcher, icon);
+          hd_switcher_menu_replace_blinking_icon (switcher, 
+			  			  hd_switcher_menu_get_icon_from_theme (icon_name,
+											AS_ICON_SIZE));
 
           icon_is_set = TRUE;
 	}
-	  
+
         if (!first_item)
 	  first_item = menu_item;
       }
+
+      g_free (icon_name);
+      g_free (summary);
+      g_free (body);
     }
     while (gtk_tree_model_iter_next (nm, &iter));
 
+    hildon_desktop_popup_menu_replace_items (switcher->priv->menu_notifications, children);
+  
+    g_list_free (children);
+    
     hd_switcher_menu_reset_notification_groups (switcher);
     
     hd_switcher_menu_scroll_to (NULL, switcher);
@@ -1824,7 +1846,8 @@ hd_switcher_menu_create_applications_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
 	  (HD_SWITCHER_MENU_ITEM (l->data));
  
     if (info != hd_wm_get_home_info (switcher->hdwm))
-      gtk_widget_destroy (GTK_WIDGET (l->data));
+      hildon_desktop_popup_menu_remove_item (switcher->priv->menu_applications,
+                                             l->data);
   }
 
   g_list_free (children);
@@ -1969,7 +1992,7 @@ hd_switcher_get_default_icon_from_entry_info (HDSwitcherMenu *switcher, HDWMEntr
     GError *error = NULL;
 
     app_pixbuf = hd_wm_entry_info_get_app_icon (info,
-                                             AS_ICON_THUMB_SIZE,
+                                             AS_ICON_SIZE,
                                              &error);
     if (error)
     {
@@ -1978,7 +2001,7 @@ hd_switcher_get_default_icon_from_entry_info (HDSwitcherMenu *switcher, HDWMEntr
       
       app_pixbuf = gtk_icon_theme_load_icon (switcher->priv->icon_theme,
                                              AS_MENU_DEFAULT_APP_ICON,
-                                             AS_ICON_THUMB_SIZE,
+                                             AS_ICON_SIZE,
                                              GTK_ICON_LOOKUP_NO_SVG,
                                              &error);
 
@@ -2007,7 +2030,7 @@ hd_switcher_menu_refresh_application_items (HDSwitcherMenu *switcher)
   {
     if (HD_IS_SWITCHER_MENU_ITEM (l->data))
     {
-      HDWMEntryInfo *info =	    
+      HDWMEntryInfo *info =
         hd_switcher_menu_item_get_entry_info (HD_SWITCHER_MENU_ITEM (l->data));
 
       hd_switcher_menu_item_set_entry_info (HD_SWITCHER_MENU_ITEM (l->data), info);
@@ -2018,7 +2041,11 @@ hd_switcher_menu_refresh_application_items (HDSwitcherMenu *switcher)
       {
         hd_switcher_menu_item_set_blinking (HD_SWITCHER_MENU_ITEM (l->data), TRUE);
       }
-    }	    
+      else if (hd_switcher_menu_item_is_blinking (HD_SWITCHER_MENU_ITEM (l->data)))
+      {
+        hd_switcher_menu_item_set_blinking (HD_SWITCHER_MENU_ITEM (l->data), FALSE);
+      }
+    }
   } 
 }
 
@@ -2039,14 +2066,10 @@ hd_switcher_menu_refresh_notification_items (HDSwitcherMenu *switcher)
 
     if (hildon_desktop_notification_manager_find_by_id (switcher->nm, id, &iter))
     {
-      g_debug ("ALOW ALOW ALOW");
-	    
       gtk_tree_model_get (GTK_TREE_MODEL (switcher->nm),
                           &iter,
                           HD_NM_COL_ACK, &ack,
                           -1);
-
-      g_debug ("ACK: %d", ack);
     }
 
     hd_switcher_menu_item_set_blinking (HD_SWITCHER_MENU_ITEM (l->data), 
@@ -2229,7 +2252,7 @@ hd_switcher_menu_show_menu_cb (HDWM *hdwm, HDSwitcherMenu *switcher)
     hildon_desktop_popup_menu_select_next_item (switcher->priv->menu_applications);
     hd_wm_debug ("selecting next item");
   }
-  else
+  else if (GTK_WIDGET_VISIBLE (GTK_BIN (switcher->priv->toggle_button)->child))
   {
     gtk_window_close_other_temporaries (GTK_WINDOW (switcher->priv->popup_window));
 
