@@ -28,6 +28,8 @@
 #include <gdk-pixbuf/gdk-pixbuf-animation.h>
 #include "hn-app-pixbuf-anim-blinker.h"
 
+#define FIXED_PERIOD 100
+
 struct _HNAppPixbufAnimBlinker
 {
         GdkPixbufAnimation parent_instance;
@@ -278,8 +280,10 @@ get_delay_time (GdkPixbufAnimationIter *anim_iter)
         HNAppPixbufAnimBlinkerIter *iter =
         	HN_APP_PIXBUF_ANIM_BLINKER_ITER (anim_iter);
         gint elapsed;
+#ifndef FIXED_PERIOD	
         gint period = iter->hn_app_pixbuf_anim_blinker->period /
         	iter->hn_app_pixbuf_anim_blinker->frequency;
+#endif	
         
         elapsed = (((iter->current_time.tv_sec - iter->start_time.tv_sec) *
         	G_USEC_PER_SEC + iter->current_time.tv_usec -
@@ -288,7 +292,13 @@ get_delay_time (GdkPixbufAnimationIter *anim_iter)
 	if (((elapsed < iter->length) ||
 	    (iter->hn_app_pixbuf_anim_blinker->length == -1)) &&
 	    (!iter->hn_app_pixbuf_anim_blinker->stopped))
+        {		
+#ifndef FIXED_PERIOD		
 		return period - (elapsed % period);
+#else		
+		return FIXED_PERIOD;
+#endif		
+        }		
 	else
 		return -1;
 }
